package kr.co.vividnext.sodalive.audio_content.detail

import android.annotation.SuppressLint
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.graphics.Rect
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.widget.RelativeLayout
import android.widget.SeekBar
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import coil.load
import coil.transform.CircleCropTransformation
import com.bumptech.glide.Glide
import com.bumptech.glide.request.RequestOptions
import com.google.gson.Gson
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audio_content.AudioContentPlayService
import kr.co.vividnext.sodalive.audio_content.comment.AudioContentCommentFragment
import kr.co.vividnext.sodalive.audio_content.modify.AudioContentModifyActivity
import kr.co.vividnext.sodalive.audio_content.order.AudioContentOrderConfirmDialog
import kr.co.vividnext.sodalive.audio_content.order.AudioContentOrderFragment
import kr.co.vividnext.sodalive.audio_content.order.OrderType
import kr.co.vividnext.sodalive.base.BaseActivity
import kr.co.vividnext.sodalive.base.SodaDialog
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.common.Utils
import kr.co.vividnext.sodalive.databinding.ActivityAudioContentDetailBinding
import kr.co.vividnext.sodalive.explorer.profile.UserProfileActivity
import kr.co.vividnext.sodalive.extensions.dpToPx
import kr.co.vividnext.sodalive.live.room.donation.LiveRoomDonationDialog
import kr.co.vividnext.sodalive.mypage.auth.Auth
import kr.co.vividnext.sodalive.mypage.auth.AuthVerifyRequest
import kr.co.vividnext.sodalive.mypage.auth.BootpayResponse
import kr.co.vividnext.sodalive.mypage.can.charge.CanChargeActivity
import kr.co.vividnext.sodalive.report.ReportType
import org.koin.android.ext.android.inject

class AudioContentDetailActivity : BaseActivity<ActivityAudioContentDetailBinding>(
    ActivityAudioContentDetailBinding::inflate
) {
    private val viewModel: AudioContentDetailViewModel by inject()

    private lateinit var loadingDialog: LoadingDialog
    private lateinit var creatorOtherContentAdapter: OtherContentAdapter
    private lateinit var sameThemeOtherContentAdapter: OtherContentAdapter

    private var audioContentId: Long = 0
    private var isAlertPreview = false
    private val audioContentReceiver = AudioContentReceiver()

    private var creatorId: Long = 0

    private var refresh = false
    private var title = ""

    @SuppressLint("SetTextI18n")
    override fun onNewIntent(intent: Intent) {
        super.onNewIntent(intent)

        binding.scrollView.scrollTo(0, 0)
        binding.sbProgress.progress = 0
        binding.ivPlayOrPause.setImageResource(0)
        binding.tvTotalDuration.text = " / 00:00:00"
        binding.tvCurrentDuration.text = "00:00:00"
        binding.rlPreviewAlert.visibility = View.GONE

        audioContentId = intent.getLongExtra(Constants.EXTRA_AUDIO_CONTENT_ID, 0)
        viewModel.getAudioContentDetail(audioContentId = audioContentId) { finish() }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        audioContentId = intent.getLongExtra(Constants.EXTRA_AUDIO_CONTENT_ID, 0)
        super.onCreate(savedInstanceState)

        if (audioContentId <= 0) {
            Toast.makeText(applicationContext, "잘못된 요청입니다.", Toast.LENGTH_LONG).show()
            finish()
        }

        bindData()
        viewModel.getAudioContentDetail(audioContentId = audioContentId) { finish() }
    }

    override fun onResume() {
        super.onResume()
        val intentFilter = IntentFilter(Constants.ACTION_AUDIO_CONTENT_RECEIVER)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            registerReceiver(audioContentReceiver, intentFilter, Context.RECEIVER_NOT_EXPORTED)
        } else {
            registerReceiver(audioContentReceiver, intentFilter)
        }

        if (refresh) {
            viewModel.getAudioContentDetail(audioContentId = audioContentId) { finish() }
        }
    }

    override fun onPause() {
        unregisterReceiver(audioContentReceiver)
        super.onPause()
    }

    override fun setupView() {
        loadingDialog = LoadingDialog(this, layoutInflater)
        binding.tvBack.text = "콘텐츠 상세"
        binding.tvBack.setOnClickListener { finish() }
        binding.ivClosePreviewAlert.setOnClickListener { viewModel.toggleShowPreviewAlert() }
        binding.ivMenu.setOnClickListener {
            showOptionMenu()
        }

        creatorOtherContentAdapter = OtherContentAdapter {
            val intent = Intent(applicationContext, AudioContentDetailActivity::class.java)
                .apply {
                    putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, it)
                    addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
                    addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP)
                }
            startActivity(intent)
        }

        binding.rvCreatorOtherContent.layoutManager = LinearLayoutManager(
            applicationContext,
            LinearLayoutManager.HORIZONTAL,
            false
        )

        binding.swipeRefreshLayout.setOnRefreshListener {
            viewModel.getAudioContentDetail(
                audioContentId = audioContentId
            ) { finish() }
            binding.swipeRefreshLayout.isRefreshing = false
        }

        binding.rvCreatorOtherContent.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 6.7f.dpToPx().toInt()
                    }

                    creatorOtherContentAdapter.itemCount - 1 -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 0
                    }

                    else -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvCreatorOtherContent.adapter = creatorOtherContentAdapter

        sameThemeOtherContentAdapter = OtherContentAdapter {
            val intent = Intent(applicationContext, AudioContentDetailActivity::class.java)
                .apply {
                    putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, it)
                    addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
                    addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP)
                }
            startActivity(intent)
        }

        binding.rvThemeOtherContent.layoutManager = LinearLayoutManager(
            applicationContext,
            LinearLayoutManager.HORIZONTAL,
            false
        )

        binding.rvThemeOtherContent.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 6.7f.dpToPx().toInt()
                    }

                    creatorOtherContentAdapter.itemCount - 1 -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 0
                    }

                    else -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvThemeOtherContent.adapter = sameThemeOtherContentAdapter

        binding.sbProgress.setOnSeekBarChangeListener(object : SeekBar.OnSeekBarChangeListener {
            override fun onProgressChanged(seekBar: SeekBar?, progress: Int, fromUser: Boolean) {
            }

            override fun onStartTrackingTouch(p0: SeekBar?) {
            }

            override fun onStopTrackingTouch(seekBar: SeekBar?) {
                if (seekBar != null) {
                    val intent = Intent(
                        this@AudioContentDetailActivity,
                        AudioContentPlayService::class.java
                    )
                    intent.action = AudioContentPlayService.MusicAction.PROGRESS.name
                    intent.putExtra(Constants.EXTRA_AUDIO_CONTENT_PROGRESS, seekBar.progress)
                    startService(intent)
                }
            }
        })

        val layoutParams = binding.ivCover.layoutParams as RelativeLayout.LayoutParams
        layoutParams.width = (screenWidth - 13.3f.dpToPx()).toInt()
        layoutParams.height = (screenWidth - 13.3f.dpToPx()).toInt()
        binding.ivCover.layoutParams = layoutParams
        binding.ivPlayLoop.setOnClickListener { viewModel.togglePlayLoop() }
        binding.llDonation.setOnClickListener {
            val dialog = LiveRoomDonationDialog(
                this,
                LayoutInflater.from(this)
            ) { can, message ->
                if (can <= 0) {
                    showToast("1캔 이상 후원하실 수 있습니다.")
                } else if (message.isBlank()) {
                    showToast("함께 보낼 메시지를 입력하세요.")
                } else {
                    donation(can, message)
                }
            }

            dialog.show(screenWidth - 26.7f.dpToPx().toInt())
        }
    }

    private fun donation(can: Int, message: String) {
        viewModel.donation(audioContentId, can, message) {
            viewModel.getAudioContentDetail(audioContentId = audioContentId) { finish() }
        }
    }

    private fun showOptionMenu() {
        val dialog = AudioContentDetailMenuBottomSheetDialog(
            isPin = viewModel.audioContentLiveData.value!!.isPin,
            isCreator = viewModel.audioContentLiveData.value!!
                .creator.creatorId == SharedPreferenceManager.userId,
            onClickPin = {
                if (viewModel.audioContentLiveData.value!!.isPin) {
                    viewModel.unPinContent(audioContentId)
                } else {
                    if (viewModel.audioContentLiveData.value!!.isAvailablePin) {
                        viewModel.pinContent(audioContentId)
                    } else {
                        SodaDialog(this@AudioContentDetailActivity,
                            layoutInflater,
                            "고정 한도 도달",
                            "이 콘텐츠를 고정하시겠어요? " +
                                "채널에 콘텐츠를 최대 3개까지 고정할 수 있습니다." +
                                "이 콘텐츠를 고정하면 가장 오래된 콘텐츠가 대체됩니다.",
                            "확인",
                            { viewModel.pinContent(audioContentId) },
                            "취소",
                            {}
                        ).show(screenWidth)
                    }
                }
            },
            onClickModify = {
                refresh = true
                setResult(RESULT_OK)
                startActivity(
                    Intent(applicationContext, AudioContentModifyActivity::class.java)
                        .apply {
                            putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, audioContentId)
                        }
                )
            },
            onClickDelete = { showDeleteDialog() },
            onClickReport = { showReportDialog() }
        )
        dialog.show(supportFragmentManager, dialog.tag)
    }

    private fun showDeleteDialog() {
        AudioContentDeleteDialog(
            this,
            layoutInflater,
            this.title,
            confirmButtonClick = {
                viewModel.deleteAudioContent(audioContentId) {
                    setResult(RESULT_OK)
                    finish()
                }
            }
        ).show(screenWidth)
    }

    private fun showReportDialog() {
        AudioContentReportDialog(this, layoutInflater) {
            viewModel.report(
                type = ReportType.AUDIO_CONTENT,
                contentId = audioContentId,
                reason = it
            )
        }.show(screenWidth)
    }

    @SuppressLint("NotifyDataSetChanged", "SetTextI18n")
    private fun bindData() {
        viewModel.toastLiveData.observe(this) {
            it?.let { Toast.makeText(applicationContext, it, Toast.LENGTH_LONG).show() }
        }

        viewModel.isLoading.observe(this) {
            if (it) {
                loadingDialog.show(screenWidth, "")
            } else {
                loadingDialog.dismiss()
            }
        }

        viewModel.isExpandDetail.observe(this) {
            binding.tvDetail.maxLines = if (it) {
                Int.MAX_VALUE
            } else {
                2
            }
        }

        viewModel.isShowPreviewAlert.observe(this) {
            binding.rlPreviewAlert.visibility = if (it) {
                View.VISIBLE
            } else {
                View.GONE
            }
        }

        viewModel.audioContentLiveData.observe(this) {
            refresh = false
            startService(
                Intent(this, AudioContentPlayService::class.java).apply {
                    action = AudioContentPlayService.MusicAction.INIT.name
                    putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, it.contentId)
                }
            )

            title = it.title
            setupCreatorArea(it.creator)
            setupMosaicArea(it.isMosaic)
            setupPlayArea(it)
            setupInfoArea(it)
            setupPurchaseButton(it)
            setupCommentArea(it)
            setupCreatorOtherContentListArea(it.creatorOtherContentList)
            setupSameThemeOtherContentList(it.sameThemeOtherContentList)
        }

        viewModel.isContentPlayLoopLiveData.observe(this) {
            if (it) {
                binding.ivPlayLoop.setImageResource(R.drawable.btn_player_repeat)
            } else {
                binding.ivPlayLoop.setImageResource(R.drawable.btn_player_repeat_done)
            }
        }
    }

    @SuppressLint("NotifyDataSetChanged")
    private fun setupSameThemeOtherContentList(
        sameThemeOtherContentList: List<OtherContentResponse>
    ) {
        if (sameThemeOtherContentList.isEmpty()) {
            binding.rvThemeOtherContent.visibility = View.GONE
            binding.llThemeOtherContentPreparing.visibility = View.VISIBLE
        } else {
            binding.rvThemeOtherContent.visibility = View.VISIBLE
            binding.llThemeOtherContentPreparing.visibility = View.GONE

            sameThemeOtherContentAdapter.items.clear()
            sameThemeOtherContentAdapter.items.addAll(sameThemeOtherContentList)
            sameThemeOtherContentAdapter.notifyDataSetChanged()
        }
    }

    @SuppressLint("NotifyDataSetChanged")
    private fun setupCreatorOtherContentListArea(
        creatorOtherContentList: List<OtherContentResponse>
    ) {
        if (creatorOtherContentList.isEmpty()) {
            binding.rvCreatorOtherContent.visibility = View.GONE
            binding.llCreatorOtherContentPreparing.visibility = View.VISIBLE
        } else {
            binding.rvCreatorOtherContent.visibility = View.VISIBLE
            binding.llCreatorOtherContentPreparing.visibility = View.GONE

            creatorOtherContentAdapter.items.clear()
            creatorOtherContentAdapter.items.addAll(creatorOtherContentList)
            creatorOtherContentAdapter.notifyDataSetChanged()
        }
    }

    private fun setupCommentArea(response: GetAudioContentDetailResponse) {
        if (response.isCommentAvailable) {
            binding.llDonation.visibility = View.VISIBLE
            binding.llComment.visibility = View.VISIBLE
            binding.tvCommentCount.text = "${response.commentCount}"

            if (response.commentCount > 0) {
                binding.ivCommentProfile.load(response.commentList[0].profileUrl) {
                    crossfade(true)
                    placeholder(R.drawable.bg_placeholder)
                    transformations(CircleCropTransformation())
                }
                binding.tvCommentText.text = response.commentList[0].comment
                binding.tvCommentText.visibility = View.VISIBLE
                binding.rlInputComment.visibility = View.GONE

                binding.llComment.setOnClickListener { showCommentBottomSheetDialog() }
            } else {
                binding.tvCommentText.visibility = View.GONE
                binding.rlInputComment.visibility = View.VISIBLE
                binding.ivCommentProfile.load(SharedPreferenceManager.profileImage) {
                    crossfade(true)
                    placeholder(R.drawable.bg_placeholder)
                    transformations(CircleCropTransformation())
                }

                binding.ivCommentSend.setOnClickListener {
                    val comment = binding.etComment.text.toString()
                    binding.etComment.setText("")
                    viewModel.registerComment(audioContentId, comment)
                }

                binding.llComment.setOnClickListener {}
            }
        } else {
            binding.llComment.visibility = View.GONE
            binding.llDonation.visibility = View.GONE
        }
    }

    private fun showCommentBottomSheetDialog() {
        val dialog = AudioContentCommentFragment(
            creatorId = creatorId,
            audioContentId = audioContentId
        )
        dialog.show(
            supportFragmentManager,
            dialog.tag
        )
    }

    private fun setupPurchaseButton(response: GetAudioContentDetailResponse) {
        if (response.releaseDate != null) {
            binding.llPurchase.visibility = View.VISIBLE
            binding.llPurchasePrice.visibility = View.GONE
            binding.tvReleaseDate.visibility = View.VISIBLE
            binding.llPurchase.background = ContextCompat.getDrawable(
                applicationContext,
                R.drawable.bg_round_corner_5_3_525252
            )

            binding.tvReleaseDate.text = response.releaseDate
        } else if (
            response.price > 0 &&
            !response.existOrdered &&
            response.orderType == null &&
            response.creator.creatorId != SharedPreferenceManager.userId
        ) {
            binding.tvReleaseDate.visibility = View.GONE
            binding.llPurchase.visibility = View.VISIBLE
            binding.llPurchasePrice.visibility = View.VISIBLE
            binding.tvPrice.text = response.price.toString()
            binding.llPurchase.background = ContextCompat.getDrawable(
                applicationContext,
                R.drawable.bg_round_corner_5_3_3bb9f1
            )

            binding.tvStrPurchaseOrRental.text = if (response.isOnlyRental) {
                " 대여하기"
            } else {
                " 구매하기"
            }

            binding.llPurchase.setOnClickListener {
                showOrderDialog(audioContent = response, isOnlyRental = response.isOnlyRental)
            }
        } else {
            binding.llPurchase.visibility = View.GONE
        }
    }

    @SuppressLint("SetTextI18n")
    private fun setupPlayArea(response: GetAudioContentDetailResponse) {
        Glide
            .with(this)
            .load(response.coverImageUrl)
            .centerCrop()
            .placeholder(R.drawable.bg_black)
            .apply(RequestOptions().override((screenWidth - 13.3f.dpToPx()).toInt()))
            .into(binding.ivCover)

        binding.ivPlayOrPause.visibility = View.GONE
        binding.tvTotalDuration.text = " / ${response.duration}"

        isAlertPreview = response.creator.creatorId != SharedPreferenceManager.userId &&
            !response.existOrdered &&
            response.price > 0

        if (
            response.releaseDate == null &&
            !isAlertPreview ||
            (isAlertPreview && response.isActivePreview)
        ) {
            binding.ivPlayOrPause.visibility = View.VISIBLE
            binding.ivPlayOrPause.setOnClickListener {
                startService(
                    Intent(this, AudioContentPlayService::class.java).apply {
                        putExtra(
                            Constants.EXTRA_AUDIO_CONTENT_COVER_IMAGE_URL,
                            response.coverImageUrl
                        )
                        putExtra(Constants.EXTRA_AUDIO_CONTENT_URL, response.contentUrl)
                        putExtra(Constants.EXTRA_NICKNAME, response.creator.nickname)
                        putExtra(Constants.EXTRA_AUDIO_CONTENT_TITLE, response.title)
                        putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, response.contentId)
                        putExtra(
                            Constants.EXTRA_AUDIO_CONTENT_CREATOR_ID,
                            response.creator.creatorId
                        )
                        putExtra(Constants.EXTRA_AUDIO_CONTENT_FREE, response.price <= 0)
                        putExtra(
                            Constants.EXTRA_AUDIO_CONTENT_PREVIEW,
                            !response.existOrdered && response.price > 0
                        )
                    }
                )
            }

            binding.ivPlayOrPause.setImageResource(
                if (!isAlertPreview) {
                    R.drawable.btn_audio_content_play
                } else {
                    R.drawable.btn_audio_content_preview_play
                }
            )
        }
    }

    @SuppressLint("SetTextI18n")
    private fun setupInfoArea(response: GetAudioContentDetailResponse) {
        binding.tvScheduledToOpen.visibility = if (response.releaseDate != null) {
            View.VISIBLE
        } else {
            View.GONE
        }
        binding.tvTheme.text = response.themeStr
        binding.tv19.visibility = if (response.isAdult) {
            View.VISIBLE
        } else {
            View.GONE
        }

        if (response.orderType != null && response.orderType == OrderType.KEEP) {
            binding.tvPurchased.visibility = View.VISIBLE
            binding.tvRental.visibility = View.GONE
            binding.tvRemainingTime.visibility = View.GONE
        } else if (response.orderType != null && response.orderType == OrderType.RENTAL) {
            binding.tvPurchased.visibility = View.GONE
            binding.tvRental.visibility = View.VISIBLE
            binding.tvRemainingTime.visibility = View.VISIBLE
            binding.tvRemainingTime.text = response.remainingTime
        } else {
            binding.tvPurchased.visibility = View.GONE
            binding.tvRental.visibility = View.GONE
            binding.tvRemainingTime.visibility = View.GONE
        }

        binding.tvTitle.text = response.title
        binding.tvDetail.text = response.detail
        binding.tvDetail.setOnClickListener { viewModel.toggleExpandDetail() }

        if (response.tag.isNotBlank()) {
            binding.tvTag.visibility = View.VISIBLE
            binding.tvTag.text = response.tag
        } else {
            binding.tvTag.visibility = View.GONE
        }

        binding.ivLike.setImageResource(
            if (response.isLike) {
                R.drawable.ic_audio_content_heart_pressed
            } else {
                R.drawable.ic_audio_content_heart_normal
            }
        )

        binding.tvLike.text = "${response.likeCount}"
        binding.llLike.setOnClickListener {
            viewModel.likeContent(contentId = audioContentId) {
                val likeCount = binding.tvLike.text.toString().toInt()
                if (it) {
                    binding.tvLike.text = "${likeCount + 1}"
                    binding.ivLike.setImageResource(R.drawable.ic_audio_content_heart_pressed)
                } else {
                    binding.tvLike.text = if (likeCount - 1 < 0) {
                        "0"
                    } else {
                        "${likeCount - 1}"
                    }
                    binding.ivLike.setImageResource(R.drawable.ic_audio_content_heart_normal)
                }
            }
        }

        binding.tvShare.setOnClickListener {
            viewModel.shareAudioContent(
                audioContentId = audioContentId,
                contentImage = response.coverImageUrl,
                contentTitle = "${response.title} - ${response.creator.nickname}"
            ) {
                val intent = Intent(Intent.ACTION_SEND)
                intent.type = "text/plain"
                intent.putExtra(Intent.EXTRA_TEXT, it)

                val shareIntent = Intent.createChooser(intent, "오디오콘텐츠 공유")
                startActivity(shareIntent)
            }
        }
    }

    private fun setupMosaicArea(isMosaic: Boolean) {
        if (isMosaic) {
            binding.alert19Bg.visibility = View.VISIBLE
            binding.llAlert19.visibility = View.VISIBLE

            binding.tvAuth.setOnClickListener {
                Auth.auth(this, applicationContext) { data ->
                    val bootpayResponse = Gson().fromJson(data, BootpayResponse::class.java)
                    val request = AuthVerifyRequest(receiptId = bootpayResponse.data.receiptId)
                    runOnUiThread {
                        viewModel.authVerify(audioContentId = audioContentId, request = request)
                    }
                }
            }
        } else {
            binding.alert19Bg.visibility = View.GONE
            binding.llAlert19.visibility = View.GONE
            binding.tvAuth.setOnClickListener {}
        }
    }

    private fun setupCreatorArea(creator: AudioContentCreator) {
        this.creatorId = creator.creatorId
        binding.rlProfile.setOnClickListener {
            startActivity(
                Intent(applicationContext, UserProfileActivity::class.java).apply {
                    putExtra(Constants.EXTRA_USER_ID, creator.creatorId)
                }
            )
        }

        binding.ivProfile.load(creator.profileImageUrl) {
            crossfade(true)
            placeholder(R.drawable.bg_placeholder)
            transformations(CircleCropTransformation())
        }

        binding.tvProfileNickname.text = creator.nickname

        if (creator.creatorId != SharedPreferenceManager.userId) {
            binding.ivFollow.visibility = View.VISIBLE

            if (creator.isFollowing) {
                binding.ivFollow.setImageResource(R.drawable.btn_notification_selected)
                binding.ivFollow.setOnClickListener {
                    viewModel.unRegisterNotification(
                        contentId = audioContentId,
                        creatorId = creator.creatorId
                    )
                }
            } else {
                binding.ivFollow.setImageResource(R.drawable.btn_notification)
                binding.ivFollow.setOnClickListener {
                    viewModel.registerNotification(
                        contentId = audioContentId,
                        creatorId = creator.creatorId
                    )
                }
            }
        } else {
            binding.ivFollow.visibility = View.GONE
        }
    }

    private fun showOrderDialog(
        audioContent: GetAudioContentDetailResponse,
        isOnlyRental: Boolean = false
    ) {
        val dialog = AudioContentOrderFragment(
            price = audioContent.price,
            isOnlyRental = isOnlyRental,
            onClickKeep = { showOrderConfirmDialog(audioContent, isOnlyRental, OrderType.KEEP) },
            onClickRental = { showOrderConfirmDialog(audioContent, isOnlyRental, OrderType.RENTAL) }
        )

        dialog.show(
            supportFragmentManager,
            dialog.tag
        )
    }

    private fun showOrderConfirmDialog(
        audioContent: GetAudioContentDetailResponse,
        isOnlyRental: Boolean = false,
        orderType: OrderType
    ) {
        AudioContentOrderConfirmDialog(
            activity = this,
            layoutInflater = layoutInflater,
            title = audioContent.title,
            theme = audioContent.themeStr,
            coverImageUrl = audioContent.coverImageUrl,
            profileImageUrl = audioContent.creator.profileImageUrl,
            nickname = audioContent.creator.nickname,
            duration = audioContent.duration,
            isOnlyRental = isOnlyRental,
            orderType = orderType,
            price = audioContent.price,
            confirmButtonClick = {
                startService(
                    Intent(this, AudioContentPlayService::class.java).apply {
                        action = AudioContentPlayService.MusicAction.CONDITIONAL_STOP.name
                        putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, audioContent.contentId)
                    }
                )

                binding.rlPreviewAlert.visibility = View.GONE

                viewModel.order(
                    contentId = audioContent.contentId,
                    orderType = orderType
                ) {
                    val intent = Intent(applicationContext, CanChargeActivity::class.java)
                    intent.putExtra(Constants.EXTRA_GO_TO_PREV_PAGE, true)
                    startActivity(intent)
                }
            },
        ).show(screenWidth)
    }

    inner class AudioContentReceiver : BroadcastReceiver() {
        @SuppressLint("SetTextI18n")
        override fun onReceive(context: Context?, intent: Intent?) {
            val nextAction = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                intent?.getSerializableExtra(
                    Constants.EXTRA_AUDIO_CONTENT_NEXT_ACTION,
                    AudioContentPlayService.MusicAction::class.java
                )
            } else {
                intent?.getSerializableExtra(Constants.EXTRA_AUDIO_CONTENT_NEXT_ACTION)
            }

            val duration = intent?.getIntExtra(Constants.EXTRA_AUDIO_CONTENT_DURATION, 0)
            val progress = intent?.getIntExtra(Constants.EXTRA_AUDIO_CONTENT_PROGRESS, 0)
            val contentId = intent?.getLongExtra(Constants.EXTRA_AUDIO_CONTENT_ID, 0)
            val isPlaying = intent?.getBooleanExtra(Constants.EXTRA_AUDIO_CONTENT_PLAYING, false)
            val changeUi = intent?.getBooleanExtra(Constants.EXTRA_AUDIO_CONTENT_CHANGE_UI, false)
            val alertPreview = intent?.getBooleanExtra(
                Constants.EXTRA_AUDIO_CONTENT_ALERT_PREVIEW,
                false
            )
            val isLoading = intent?.getBooleanExtra(
                Constants.EXTRA_AUDIO_CONTENT_LOADING,
                false
            )

            viewModel.isLoading.value = isLoading ?: false

            if (this@AudioContentDetailActivity.audioContentId == contentId) {
                runOnUiThread {
                    if (changeUi != null && changeUi) {
                        binding.ivPlayOrPause.setImageResource(
                            if (isPlaying != null && isPlaying) {
                                R.drawable.btn_audio_content_pause
                            } else {
                                if (isAlertPreview) {
                                    R.drawable.btn_audio_content_preview_play
                                } else {
                                    R.drawable.btn_audio_content_play
                                }
                            }
                        )
                    }
                }

                if (duration != null && duration > 0) {
                    binding.sbProgress.max = duration
                    binding.tvTotalDuration.text = " / ${Utils.convertDurationToString(duration)}"
                }

                if (progress != null && progress > 0) {
                    binding.sbProgress.progress = progress
                    binding.tvCurrentDuration.text = Utils.convertDurationToString(progress)
                }

                if (alertPreview != null && alertPreview && isAlertPreview) {
                    viewModel.toggleShowPreviewAlert()
                }

                if (nextAction != null) {
                    startService(
                        Intent(
                            this@AudioContentDetailActivity,
                            AudioContentPlayService::class.java
                        ).apply {
                            action = (nextAction as AudioContentPlayService.MusicAction).name
                        }
                    )
                }
            }
        }
    }
}
